<?php
if (!defined('__TYPECHO_ROOT_DIR__')) exit;

/**
 * OpenAI写作插件
 * @package AiContent
 * @author Memory
 * @version 3.2.0
 * @link https://lvtao.net
 */
class AiContent_Plugin implements Typecho_Plugin_Interface
{
    public static function activate()
    {
        Typecho_Plugin::factory('admin/write-post.php')->aiWrite = array('AiContent_Plugin', 'render');
    }

    public static function deactivate()
    {
    }

    public static function config(Typecho_Widget_Helper_Form $form)
    {
        $keyValue = new Typecho_Widget_Helper_Form_Element_Text(
            'keyValue',
            NULL,
            "sk-xxx",
            _t('API KEY'),
            _t('输入调用API的key')
        );
        $form->addInput($keyValue);

        // 配置API地址
        $apiUrl = new Typecho_Widget_Helper_Form_Element_Text(
            'apiUrl',
            NULL,
            "https://api.openai.com",
            _t('API地址'),
            _t('请输入API地址，不要省略（https://）或（http://）不要带有（/v1）')
        );
        $form->addInput($apiUrl);

        // 摘要最大长度
        $maxLength = new Typecho_Widget_Helper_Form_Element_Text(
            'maxLength',
            NULL,
            '8000',
            _t('最大长度'),
            _t('输出的最大文字数量。')
        );
        $form->addInput($maxLength);
        
        // 提示词
        $prompt = new Typecho_Widget_Helper_Form_Element_Text(
            'prompt',
            NULL,
            '你是一个智能文章生成助手',
            _t('提示词'),
            _t('可以理解为给AI 模型的指令')
        );
        $form->addInput($prompt);
        
        $model = new Typecho_Widget_Helper_Form_Element_Text(
            'model',
            NULL,
            'gpt-3.5-turbo',
            _t('对话模型'),
            _t('请根据自己的接口配置，OpenAI推荐：gpt-3.5-turbo，gpt-4-turbo，gpt-4o等')
        );
        $form->addInput($model);
        
        $temperature = new Typecho_Widget_Helper_Form_Element_Text(
            'temperature',
            NULL,
            '0.7',
            _t('创造性'),
            _t('用于调节模型输出的概率分布，范围0-1，0模型倾向于选择概率最高的词，这样生成的文本更稳定、更可预测，但可能缺乏多样性；1模型更有可能选择概率较低的词，这会增加文本的多样性和创造性，但同时也可能导致生成的文本质量下降')
        );
        $form->addInput($temperature);
    }

    public static function personalConfig(Typecho_Widget_Helper_Form $form)
    {
    }

    public static function render()
    {
        $apiUrl = rtrim(Helper::options()->plugin('AiContent')->apiUrl, '/');
        $keyValue = Helper::options()->plugin('AiContent')->keyValue;
        $maxLength = Helper::options()->plugin('AiContent')->maxLength;
        $prompt = Helper::options()->plugin('AiContent')->prompt;
        $model = Helper::options()->plugin('AiContent')->model;
        $temperature = Helper::options()->plugin('AiContent')->temperature;
        
        echo <<<EOF
            <script type="text/javascript">
                function showAiContent(){
                  if(document.getElementById("AiWriteAsk").style.display=="none"){
                      document.getElementById("AiWriteAsk").style.display="flex"; 
                      document.getElementById("aiWrite-button").innerHTML="收起AI写作"; 
                  } else {
                      document.getElementById("AiWriteAsk").style.display="none";
                      document.getElementById("aiWrite-button").innerHTML="展开AI写作"; 
                  }
               }

               function startWrite() {
                    // 显示遮罩
                    coverDiv();

                    var keywords = document.getElementById('keywords').value;
                    var xhr = new XMLHttpRequest();
                    xhr.open('POST', '{$apiUrl}/v1/chat/completions', true);
                    xhr.setRequestHeader('Content-Type', 'application/json');
                    xhr.setRequestHeader('Authorization', 'Bearer {$keyValue}');

                    var data = JSON.stringify({
                        "model": "{$model}",
                        "messages": [
                            {"role": "system", "content": "{$prompt}"},
                            {"role": "user", "content": keywords}
                        ],
                        "temperature": {$temperature},
                        "max_tokens": {$maxLength}
                    });

                    xhr.onreadystatechange = function () {
                        if (xhr.readyState === 4 && xhr.status === 200) {
                            var response = JSON.parse(xhr.responseText);
                            var text = response.choices[0].message.content;
                            document.getElementById('text').value = text;
                        }
                        hide(); // 隐藏遮罩
                    };

                    xhr.send(data);
               }

               function coverDiv() {
                    // 显示生成中的遮罩
                    var procbg = document.createElement("div");
                    procbg.setAttribute("id", "mybg");
                    procbg.style.background = "#000000";
                    procbg.style.width = "100%";
                    procbg.style.height = "100%";
                    procbg.style.position = "fixed";
                    procbg.style.top = "0";
                    procbg.style.left = "0";
                    procbg.style.zIndex = "500";
                    procbg.style.opacity = "0.6";

                    var textDiv = document.createElement("div");
                    textDiv.innerHTML = "正在生成中...";
                    textDiv.style.color = "#ffffff";
                    textDiv.style.fontSize = "24px";
                    textDiv.style.position = "absolute";
                    textDiv.style.top = "50%";
                    textDiv.style.left = "50%";
                    textDiv.style.transform = "translate(-50%, -50%)";

                    procbg.appendChild(textDiv);
                    document.body.appendChild(procbg);
               }

               function hide() {
                    var mybg = document.getElementById("mybg");
                    if (mybg) {
                        document.body.removeChild(mybg);
                    }
               }
            </script>

            <div style="margin:0px 0px 10px 0px;display:flex;">
                <div id="AiWriteAsk" style="display:none;flex:1;">
                    <input type="text" placeholder="请输入关键词..." class="text" id="keywords" style="flex:1"/>
                    <label onclick="startWrite()" class="primary" style="text-decoration:none;color:white;padding:7px;margin-left:5px;margin-right:5px">生成内容</label>
                </div>
                <a class="primary" id="aiWrite-button" style="text-decoration:none;color:white;padding:7px;" onclick="showAiContent()">展开AI写作</a>
            </div>
EOF;
    }
}